@@ -25,41 +25,41 @@ import importlib.util
 import json
 import logging
 import os
+import re
 import sys
 import zipfile
 
-from schema import (Optional, SchemaError, SchemaMissingKeyError,
-                    SchemaForbiddenKeyError, SchemaUnexpectedTypeError)
+from ruamel.yaml import YAML, parser as YAMLParser, scanner as YAMLScanner
+from schema import (Optional, SchemaError, SchemaWrongKeyError,
+                    SchemaMissingKeyError, SchemaForbiddenKeyError,
+                    SchemaUnexpectedTypeError)
 import boto3
-import yaml
 
-from panther_analysis_tool.schemas import TYPE_SCHEMA, GLOBAL_SCHEMA, POLICY_SCHEMA, RULE_SCHEMA
+from panther_analysis_tool.schemas import (TYPE_SCHEMA, DATA_MODEL_SCHEMA,
+                                           GLOBAL_SCHEMA, POLICY_SCHEMA,
+                                           RULE_SCHEMA)
+from panther_analysis_tool.test_case import DataModel, TestCase
 
+DATA_MODEL_LOCATION = './data_models'
 HELPERS_LOCATION = './global_helpers'
 
+DATA_MODEL_PATH_PATTERN = '*data_models*'
 HELPERS_PATH_PATTERN = '*/global_helpers'
 RULES_PATH_PATTERN = '*rules*'
 POLICIES_PATH_PATTERN = '*policies*'
 
+DATAMODEL = 'datamodel'
+GLOBAL = 'global'
+RULE = 'rule'
+POLICY = 'policy'
 
-class TestCase():
 
-    def __init__(self, data: Dict[str, Any]) -> None:
-        """
-        Args:
-            data (Dict[str, Any]): An AWS Resource representation or Log event to test the policy
-            or rule against respectively.
-        """
-        self._data = data
+# exception for conflicting ids
+class AnalysisIDConflictException(Exception):
 
-    def __getitem__(self, arg: str) -> Any:
-        return self._data.get(arg, None)
-
-    def __iter__(self) -> Iterator:
-        return iter(self._data)
-
-    def get(self, arg: str, default: Any = None) -> Any:
-        return self._data.get(arg, default)
+    def __init__(self, analysis_id: str):
+        self.message = 'Conflicting AnalysisID: [{}]'.format(analysis_id)
+        super().__init__(self.message)
 
 
 def load_module(filename: str) -> Tuple[Any, Any]:
@@ -78,7 +78,7 @@ def load_module(filename: str) -> Tuple[Any, Any]:
         assert isinstance(spec.loader, Loader)  #nosec
         spec.loader.exec_module(module)
     except FileNotFoundError as err:
-        print('\t[ERROR] File not found, skipping\n')
+        print('\t[ERROR] File not found: ' + filename + ', skipping\n')
         return None, err
     except Exception as err:  # pylint: disable=broad-except
         # Catch arbitrary exceptions thrown by user code
@@ -87,7 +87,8 @@ def load_module(filename: str) -> Tuple[Any, Any]:
     return module, None
 
 
-def load_analysis_specs(directory: str) -> Iterator[Tuple[str, str, Any]]:
+def load_analysis_specs(
+        directories: List[str]) -> Iterator[Tuple[str, str, Any, Any]]:
     """Loads the analysis specifications from a file.
 
     Args:
@@ -96,27 +97,49 @@ def load_analysis_specs(directory: str) -> Iterator[Tuple[str, str, Any]]:
     Yields:
         A tuple of the relative filepath, directory name, and loaded analysis specification dict.
     """
-    for relative_path, _, file_list in os.walk(directory):
-        # If the user runs with no path/out args, filter to make sure
-        # we only run folders with valid analysis files.
-        if directory == '.':
-            if not any([
-                    fnmatch(relative_path, path_pattern)
-                    for path_pattern in (HELPERS_PATH_PATTERN,
-                                         RULES_PATH_PATTERN,
-                                         POLICIES_PATH_PATTERN)
-            ]):
-                logging.debug('Skipping path %s', relative_path)
-                continue
-        for filename in sorted(file_list):
-            spec_filename = os.path.join(relative_path, filename)
-            if fnmatch(filename, '*.y*ml'):
-                with open(spec_filename, 'r') as spec_file_obj:
-                    yield spec_filename, relative_path, yaml.safe_load(
-                        spec_file_obj)
-            if fnmatch(filename, '*.json'):
-                with open(spec_filename, 'r') as spec_file_obj:
-                    yield spec_filename, relative_path, json.load(spec_file_obj)
+    # setup a list of paths to ensure we do not import the same files
+    # multiple times, which can happen when testing from root directory without filters
+    loaded_specs = []
+    for directory in directories:
+        for relative_path, _, file_list in os.walk(directory):
+            # setup yaml object
+            yaml = YAML(typ='safe')
+            # If the user runs with no path args, filter to make sure
+            # we only run folders with valid analysis files. Ensure we test
+            # files in the current directory by not skipping this iteration
+            # when relative_path is the current dir
+            if directory in ['.', './'] and relative_path not in ['.', './']:
+                if not any([
+                        fnmatch(relative_path, path_pattern)
+                        for path_pattern in (
+                            DATA_MODEL_PATH_PATTERN, HELPERS_PATH_PATTERN,
+                            RULES_PATH_PATTERN, POLICIES_PATH_PATTERN)
+                ]):
+                    logging.debug('Skipping path %s', relative_path)
+                    continue
+            for filename in sorted(file_list):
+                spec_filename = os.path.join(relative_path, filename)
+                # skip loading files that have already been imported
+                if spec_filename in loaded_specs:
+                    continue
+                loaded_specs.append(spec_filename)
+                if fnmatch(filename, '*.y*ml'):
+                    with open(spec_filename, 'r') as spec_file_obj:
+                        try:
+                            yield spec_filename, relative_path, yaml.load(
+                                spec_file_obj), None
+                        except (YAMLParser.ParserError,
+                                YAMLScanner.ScannerError) as err:
+                            # recreate the yaml object and yeild the error
+                            yaml = YAML(typ='safe')
+                            yield spec_filename, relative_path, None, err
+                if fnmatch(filename, '*.json'):
+                    with open(spec_filename, 'r') as spec_file_obj:
+                        try:
+                            yield spec_filename, relative_path, json.load(
+                                spec_file_obj), None
+                        except ValueError as err:
+                            yield spec_filename, relative_path, None, err
 
 
 def datetime_converted(obj: Any) -> Any:
@@ -144,10 +167,10 @@ def zip_analysis(args: argparse.Namespace) -> Tuple[int, str]:
     Returns:
         A tuple of return code and the archive filename.
     """
-    return_code, _ = test_analysis(args)
-
-    if return_code == 1:
-        return return_code, ''
+    if not args.skip_tests:
+        return_code, _ = test_analysis(args)
+        if return_code == 1:
+            return return_code, ''
 
     logging.info('Zipping analysis packs in %s to %s', args.path, args.out)
     # example: 2019-08-05T18-23-25
@@ -156,11 +179,12 @@ def zip_analysis(args: argparse.Namespace) -> Tuple[int, str]:
         ':', '-')
     filename = 'panther-analysis-{}.zip'.format(current_time)
     with zipfile.ZipFile(filename, 'w', zipfile.ZIP_DEFLATED) as zip_out:
-        # Always zip the helpers
+        # Always zip the helpers and data models
         analysis = []
         files: Set[str] = set()
-        for (file_name, f_path, spec) in list(load_analysis_specs(
-                args.path)) + list(load_analysis_specs(HELPERS_LOCATION)):
+        for (file_name, f_path, spec, _) in list(
+                load_analysis_specs(
+                    [args.path, HELPERS_LOCATION, DATA_MODEL_LOCATION])):
             if file_name not in files:
                 analysis.append((file_name, f_path, spec))
                 files.add(file_name)
@@ -168,7 +192,9 @@ def zip_analysis(args: argparse.Namespace) -> Tuple[int, str]:
         analysis = filter_analysis(analysis, args.filter)
         for analysis_spec_filename, dir_name, analysis_spec in analysis:
             zip_out.write(analysis_spec_filename)
-            zip_out.write(os.path.join(dir_name, analysis_spec['Filename']))
+            # datamodels may not have python body
+            if 'Filename' in analysis_spec:
+                zip_out.write(os.path.join(dir_name, analysis_spec['Filename']))
 
     return 0, filename
 
@@ -188,23 +214,24 @@ def upload_analysis(args: argparse.Namespace) -> Tuple[int, str]:
     if return_code == 1:
         return return_code, ''
 
+    # optionally set env variable for profile passed as argument
+    # this must be called prior to setting up the client
+    if args.aws_profile is not None:
+        logging.info('Using AWS profile: %s', args.aws_profile)
+        set_env("AWS_PROFILE", args.aws_profile)
+
     client = boto3.client('lambda')
 
     with open(archive, 'rb') as analysis_zip:
         zip_bytes = analysis_zip.read()
         payload = {
-            'resource':
-                '/upload',
-            'HTTPMethod':
-                'POST',
-            'Body':
-                json.dumps({
-                    'Data': base64.b64encode(zip_bytes).decode('utf-8'),
-                    # The UserID is required by Panther for this API call, but we have no way of
-                    # acquiring it and it isn't used for anything. This is a valid UUID used by the
-                    # Panther deployment tool to indicate this action was performed automatically.
-                    'UserID': '00000000-0000-4000-8000-000000000000',
-                }),
+            'bulkUpload': {
+                'data': base64.b64encode(zip_bytes).decode('utf-8'),
+                # The UserID is required by Panther for this API call, but we have no way of
+                # acquiring it and it isn't used for anything. This is a valid UUID used by the
+                # Panther deployment tool to indicate this action was performed automatically.
+                'userId': '00000000-0000-4000-8000-000000000000',
+            },
         }
 
         logging.info('Uploading pack to Panther')
@@ -216,11 +243,39 @@ def upload_analysis(args: argparse.Namespace) -> Tuple[int, str]:
         response_str = response['Payload'].read().decode('utf-8')
         response_payload = json.loads(response_str)
 
-        if response_payload['statusCode'] != 200:
-            logging.warning(
-                'Failed to upload to Panther\n\tstatus code: %s\n\terror message: %s',
-                response_payload['statusCode'], response_payload['body'])
-            return 1, ''
+        if response_payload.get(
+                'errorMessage') or response_payload.get('statusCode') != 200:
+            # try deprecated version, using analysis-api gateway
+            deprecated_payload = {
+                'resource':
+                    '/upload',
+                'HTTPMethod':
+                    'POST',
+                'Body':
+                    json.dumps({
+                        'Data': base64.b64encode(zip_bytes).decode('utf-8'),
+                        # The UserID is required by Panther for this API call, but we have
+                        # no way of acquiring it and it isn't used for anything. This is a
+                        # valid UUID used by the Panther deployment tool to indicate this
+                        # action was performed automatically.
+                        'UserID': '00000000-0000-4000-8000-000000000000',
+                    }),
+            }
+            response = client.invoke(FunctionName='panther-analysis-api',
+                                     InvocationType='RequestResponse',
+                                     LogType='None',
+                                     Payload=json.dumps(deprecated_payload))
+            response_str = response['Payload'].read().decode('utf-8')
+            response_payload = json.loads(response_str)
+
+            if response_payload.get('errorMessage') or response_payload.get(
+                    'statusCode') != 200:
+                logging.warning(
+                    'Failed to upload to Panther\n\tstatus code: %s\n\terror message: %s',
+                    response_payload.get('statusCode', 0),
+                    response_payload.get('errorMessage',
+                                         response_payload.get('body')))
+                return 1, ''
 
         body = json.loads(response_payload['body'])
         logging.info('Upload success.')
@@ -240,50 +295,109 @@ def test_analysis(args: argparse.Namespace) -> Tuple[int, list]:
         A tuple of the return code, and a list of tuples containing invalid specs and their error.
     """
     failed_tests: DefaultDict[str, list] = defaultdict(list)
-    tests: List[str] = []
     logging.info('Testing analysis packs in %s\n', args.path)
 
-    # First classify each file
-    global_analysis, analysis, invalid_specs = classify_analysis(
-        list(load_analysis_specs(args.path)) +
-        list(load_analysis_specs(HELPERS_LOCATION)))
+    # First classify each file, always include globals and data models location
+    data_models, global_analysis, analysis, invalid_specs = classify_analysis(
+        list(
+            load_analysis_specs(
+                [args.path, HELPERS_LOCATION, DATA_MODEL_LOCATION])))
 
-    if len(analysis) == 0 and len(global_analysis) == 0:
+    if all(len(x) == 0 for x in [data_models, global_analysis, analysis]):
+        if len(invalid_specs) > 0:
+            return 1, invalid_specs
         return 1, ["Nothing to test in {}".format(args.path)]
 
     # Apply the filters as needed
+    data_models = filter_analysis(data_models, args.filter)
     global_analysis = filter_analysis(global_analysis, args.filter)
     analysis = filter_analysis(analysis, args.filter)
 
-    if len(analysis) == 0 and len(global_analysis) == 0:
+    if all(len(x) == 0 for x in [data_models, global_analysis, analysis]):
         return 1, [
             "No analyses in {} matched filters {}".format(
                 args.path, args.filter)
         ]
 
-    # First import the globals
+    # import each data model, global, policy, or rule and run its tests
+    # first import the globals
+    #   add them sys.modules to be used by rule and/or policies tests
+    invalid_globals = setup_global_helpers(global_analysis)
+    invalid_specs.extend(invalid_globals)
+
+    # then, setup data model dictionary to be used in rule/policy tests
+    log_type_to_data_model, invalid_data_models = setup_data_models(data_models)
+    invalid_specs.extend(invalid_data_models)
+
+    # then, import rules and policies; run tests
+    failed_tests, invalid_detection = setup_run_tests(log_type_to_data_model,
+                                                      analysis,
+                                                      args.minimum_tests)
+    invalid_specs.extend(invalid_detection)
+
+    print_summary(args.path, len(analysis), failed_tests, invalid_specs)
+    return int(bool(failed_tests or invalid_specs)), invalid_specs
+
+
+def setup_global_helpers(global_analysis: List[Any]) -> List[Any]:
+    invalid_specs = []
     for analysis_spec_filename, dir_name, analysis_spec in global_analysis:
+        analysis_id = analysis_spec['GlobalID']
         module, load_err = load_module(
             os.path.join(dir_name, analysis_spec['Filename']))
         # If the module could not be loaded, continue to the next
         if load_err:
             invalid_specs.append((analysis_spec_filename, load_err))
             break
-        sys.modules[analysis_spec['GlobalID']] = module
+        sys.modules[analysis_id] = module
+    return invalid_specs
 
-    # Next import each policy or rule and run its tests
+
+def setup_data_models(
+        data_models: List[Any]) -> Tuple[Dict[str, DataModel], List[Any]]:
+    invalid_specs = []
+    # log_type_to_data_model is a dict used to map LogType to a unique
+    # data model, ensuring there is at most one DataModel per LogType
+    log_type_to_data_model: Dict[str, DataModel] = dict()
+    for analysis_spec_filename, dir_name, analysis_spec in data_models:
+        analysis_id = analysis_spec['DataModelID']
+        if analysis_spec['Enabled']:
+            # load optional python modules
+            module = None
+            if 'Filename' in analysis_spec:
+                module, load_err = load_module(
+                    os.path.join(dir_name, analysis_spec['Filename']))
+                # If the module could not be loaded, continue to the next
+                if load_err:
+                    invalid_specs.append((analysis_spec_filename, load_err))
+                    continue
+                sys.modules[analysis_id] = module
+            # setup the mapping lookups
+            data_model = DataModel(analysis_id, analysis_spec['Mappings'],
+                                   module)
+            # check if the LogType already has an enabled data model
+            for log_type in analysis_spec['LogTypes']:
+                if log_type in log_type_to_data_model:
+                    print('\t[ERROR] Conflicting Enabled LogTypes\n')
+                    invalid_specs.append(
+                        (analysis_spec_filename,
+                         'Conflicting Enabled LogType [{}] in Data Model [{}]'.
+                         format(log_type, analysis_id)))
+                    continue
+                log_type_to_data_model[log_type] = data_model
+    return log_type_to_data_model, invalid_specs
+
+
+def setup_run_tests(
+        log_type_to_data_model: Dict[str, DataModel], analysis: List[Any],
+        minimum_tests: int) -> Tuple[DefaultDict[str, List[Any]], List[Any]]:
+    invalid_specs = []
+    failed_tests: DefaultDict[str, list] = defaultdict(list)
     for analysis_spec_filename, dir_name, analysis_spec in analysis:
+        analysis_type = analysis_spec['AnalysisType']
         analysis_id = analysis_spec.get('PolicyID') or analysis_spec['RuleID']
         print(analysis_id)
 
-        # Check if the AnalysisID has already been loaded
-        if analysis_id in tests:
-            print('\t[ERROR] Conflicting AnalysisID\n')
-            invalid_specs.append(
-                (analysis_spec_filename,
-                 'Conflicting AnalysisID: {}'.format(analysis_id)))
-            continue
-
         module, load_err = load_module(
             os.path.join(dir_name, analysis_spec['Filename']))
         # If the module could not be loaded, continue to the next
@@ -291,22 +405,21 @@ def test_analysis(args: argparse.Namespace) -> Tuple[int, list]:
             invalid_specs.append((analysis_spec_filename, load_err))
             continue
 
-        tests.append(analysis_id)
         analysis_funcs = {}
-        if analysis_spec['AnalysisType'] == 'policy':
+        if analysis_type == POLICY:
             analysis_funcs['run'] = module.policy
-        elif analysis_spec['AnalysisType'] == 'rule':
+        elif analysis_type == RULE:
             analysis_funcs['run'] = module.rule
             if 'dedup' in dir(module):
                 analysis_funcs['dedup'] = module.dedup
             if 'title' in dir(module):
                 analysis_funcs['title'] = module.title
 
-        failed_tests = run_tests(analysis_spec, analysis_funcs, failed_tests)
+        failed_tests = run_tests(analysis_spec, analysis_funcs,
+                                 log_type_to_data_model, failed_tests,
+                                 minimum_tests)
         print('')
-
-    print_summary(args.path, len(analysis), failed_tests, invalid_specs)
-    return int(bool(failed_tests or invalid_specs)), invalid_specs
+    return failed_tests, invalid_specs
 
 
 def print_summary(test_path: str, num_tests: int, failed_tests: Dict[str, list],
@@ -362,28 +475,62 @@ def filter_analysis(analysis: List[Any], filters: Dict[str, List]) -> List[Any]:
 
 
 def classify_analysis(
-    specs: List[Tuple[str, str,
-                      Any]]) -> Tuple[List[Any], List[Any], List[Any]]:
+    specs: List[Tuple[str, str, Any, Any]]
+) -> Tuple[List[Any], List[Any], List[Any], List[Any]]:
+
     # First determine the type of each file
+    data_models = []
     global_analysis = []
     analysis = []
     invalid_specs = []
+    # each analysis type must have a unique id, track used ids and
+    # add any duplicates to the invalid_specs
+    analysis_ids = []
 
-    for analysis_spec_filename, dir_name, analysis_spec in specs:
+    for analysis_spec_filename, dir_name, analysis_spec, error in specs:
+        keys = list()
         try:
+            # check for parsing errors from json.loads (ValueError) / yaml.safe_load (YAMLError)
+            if error:
+                raise error
+            # validate the schema
             TYPE_SCHEMA.validate(analysis_spec)
+            if analysis_spec['AnalysisType'] == 'datamodel':
+                keys = list(DATA_MODEL_SCHEMA.schema.keys())
+                DATA_MODEL_SCHEMA.validate(analysis_spec)
+                if analysis_spec['DataModelID'] in analysis_ids:
+                    raise AnalysisIDConflictException(
+                        analysis_spec['DataModelID'])
+                analysis_ids.append(analysis_spec['DataModelID'])
+                data_models.append(
+                    (analysis_spec_filename, dir_name, analysis_spec))
+            if analysis_spec['AnalysisType'] == 'global':
+                keys = list(GLOBAL_SCHEMA.schema.keys())
+                GLOBAL_SCHEMA.validate(analysis_spec)
+                if analysis_spec['GlobalID'] in analysis_ids:
+                    raise AnalysisIDConflictException(analysis_spec['GlobalID'])
+                analysis_ids.append(analysis_spec['GlobalID'])
+                global_analysis.append(
+                    (analysis_spec_filename, dir_name, analysis_spec))
             if analysis_spec['AnalysisType'] == 'policy':
+                keys = list(POLICY_SCHEMA.schema.keys())
                 POLICY_SCHEMA.validate(analysis_spec)
+                if analysis_spec['PolicyID'] in analysis_ids:
+                    raise AnalysisIDConflictException(analysis_spec['PolicyID'])
+                analysis_ids.append(analysis_spec['PolicyID'])
                 analysis.append(
                     (analysis_spec_filename, dir_name, analysis_spec))
             if analysis_spec['AnalysisType'] == 'rule':
+                keys = list(RULE_SCHEMA.schema.keys())
                 RULE_SCHEMA.validate(analysis_spec)
+                if analysis_spec['RuleID'] in analysis_ids:
+                    raise AnalysisIDConflictException(analysis_spec['RuleID'])
+                analysis_ids.append(analysis_spec['RuleID'])
                 analysis.append(
                     (analysis_spec_filename, dir_name, analysis_spec))
-            if analysis_spec['AnalysisType'] == 'global':
-                GLOBAL_SCHEMA.validate(analysis_spec)
-                global_analysis.append(
-                    (analysis_spec_filename, dir_name, analysis_spec))
+        except SchemaWrongKeyError as err:
+            invalid_specs.append(
+                (analysis_spec_filename, handle_wrong_key_error(err, keys)))
         except (SchemaError, SchemaMissingKeyError, SchemaForbiddenKeyError,
                 SchemaUnexpectedTypeError) as err:
             invalid_specs.append((analysis_spec_filename, err))
@@ -393,11 +540,31 @@ def classify_analysis(
             invalid_specs.append((analysis_spec_filename, err))
             continue
 
-    return (global_analysis, analysis, invalid_specs)
+    return (data_models, global_analysis, analysis, invalid_specs)
+
+
+def handle_wrong_key_error(err: SchemaWrongKeyError, keys: list) -> Exception:
+    regex = r"Wrong key(?:s)? (.+?) in (.*)$"
+    matches = re.match(regex, str(err))
+    msg = '{} not in list of valid keys: {}'
+    try:
+        if matches:
+            raise SchemaWrongKeyError(msg.format(matches.group(1),
+                                                 keys)) from err
+        raise SchemaWrongKeyError(msg.format('UNKNOWN_KEY', keys)) from err
+    except SchemaWrongKeyError as exc:
+        return exc
 
 
 def run_tests(analysis: Dict[str, Any], analysis_funcs: Dict[str, Any],
-              failed_tests: DefaultDict[str, list]) -> DefaultDict[str, list]:
+              analysis_data_models: Dict[str, DataModel],
+              failed_tests: DefaultDict[str, list],
+              minimum_tests: int) -> DefaultDict[str, list]:
+
+    if len(analysis.get('Tests', [])) < minimum_tests:
+        failed_tests[analysis.get('PolicyID') or analysis['RuleID']].append(
+            'Insufficient test coverage: {} tests required but only {} found'.
+            format(minimum_tests, len(analysis.get('Tests', []))))
 
     # First check if any tests exist, so we can print a helpful message if not
     if 'Tests' not in analysis:
@@ -407,7 +574,10 @@ def run_tests(analysis: Dict[str, Any], analysis_funcs: Dict[str, Any],
 
     for unit_test in analysis['Tests']:
         try:
-            test_case = TestCase(unit_test.get('Resource') or unit_test['Log'])
+            entry = unit_test.get('Resource') or unit_test['Log']
+            log_type = entry.get('p_log_type', '')
+            # set up each test case, including any relevant data models
+            test_case = TestCase(entry, analysis_data_models.get(log_type))
             result = analysis_funcs['run'](test_case)
         except KeyError as err:
             logging.warning('KeyError: {%s}', err)
@@ -449,6 +619,13 @@ def run_tests(analysis: Dict[str, Any], analysis_funcs: Dict[str, Any],
                 print('\t\t[{}] [{}] {}'.format(
                     test_result[func], func, analysis_funcs[func](test_case)))
 
+    if minimum_tests > 1 and not (
+        [x for x in analysis['Tests'] if x['ExpectedResult']] and
+        [x for x in analysis['Tests'] if not x['ExpectedResult']]):
+        failed_tests[analysis.get('PolicyID') or analysis['RuleID']].append(
+            'Insufficient test coverage: expected at least one positive and one negative test'
+        )
+
     return failed_tests
 
 
@@ -459,7 +636,7 @@ def setup_parser() -> argparse.ArgumentParser:
         prog='panther_analysis_tool')
     parser.add_argument('--version',
                         action='version',
-                        version='panther_analysis_tool 0.3.5')
+                        version='panther_analysis_tool 0.4.2')
     subparsers = parser.add_subparsers()
 
     test_parser = subparsers.add_parser(
@@ -475,6 +652,15 @@ def setup_parser() -> argparse.ArgumentParser:
                              required=False,
                              metavar="KEY=VALUE",
                              nargs='+')
+    test_parser.add_argument(
+        '--minimum-tests',
+        default='0',
+        type=int,
+        help=
+        'The minimum number of tests in order for a detection to be considered passing. If a number'
+        +
+        'greater than 1 is specified, at least one True and one False test is required.',
+        required=False)
     test_parser.add_argument('--debug', action='store_true', dest='debug')
     test_parser.set_defaults(func=test_analysis)
 
@@ -499,12 +685,30 @@ def setup_parser() -> argparse.ArgumentParser:
                             required=False,
                             metavar="KEY=VALUE",
                             nargs='+')
+    zip_parser.add_argument(
+        '--minimum-tests',
+        default='0',
+        type=int,
+        help=
+        'The minimum number of tests in order for a detection to be considered passing. If a number'
+        +
+        'greater than 1 is specified, at least one True and one False test is required.',
+        required=False)
     zip_parser.add_argument('--debug', action='store_true', dest='debug')
+    zip_parser.add_argument('--skip-tests',
+                            action='store_true',
+                            dest='skip_tests')
     zip_parser.set_defaults(func=zip_analysis)
 
     upload_parser = subparsers.add_parser(
         'upload',
         help='Upload specified policies and rules to a Panther deployment.')
+    upload_parser.add_argument(
+        '--aws-profile',
+        type=str,
+        help=
+        'The AWS profile to use when uploading to an AWS Panther deployment.',
+        required=False)
     upload_parser.add_argument(
         '--path',
         default='.',
@@ -518,11 +722,23 @@ def setup_parser() -> argparse.ArgumentParser:
         help=
         'The location to store a local copy of the packaged policies and rules.',
         required=False)
+    upload_parser.add_argument(
+        '--minimum-tests',
+        default='0',
+        type=int,
+        help=
+        'The minimum number of tests in order for a detection to be considered passing. If a number'
+        +
+        'greater than 1 is specified, at least one True and one False test is required.',
+        required=False)
     upload_parser.add_argument('--filter',
                                required=False,
                                metavar="KEY=VALUE",
                                nargs='+')
     upload_parser.add_argument('--debug', action='store_true', dest='debug')
+    upload_parser.add_argument('--skip-tests',
+                               action='store_true',
+                               dest='skip_tests')
     upload_parser.set_defaults(func=upload_analysis)
 
     return parser
@@ -551,9 +767,14 @@ def parse_filter(filters: List[str]) -> Dict[str, Any]:
     return parsed_filters
 
 
+def set_env(key: str, value: str) -> None:
+    os.environ[key] = value
+
+
 def run() -> None:
     parser = setup_parser()
-    args = parser.parse_args()
+    # if no args are passed, print the help output
+    args = parser.parse_args(args=None if sys.argv[1:] else ['--help'])
 
     logging.basicConfig(format='[%(levelname)s]: %(message)s',
                         level=logging.DEBUG if args.debug else logging.INFO)
